/*
 * Low-level PXA250/210 sleep/wakeUp support
 *
 * Initial SA1110 code:
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * Adapted for PXA by Nicolas Pitre:
 * Copyright (c) 2002 Monta Vista Software, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 */

#include <linux/config.h>
#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/arch/pm.h>

		.text

/*
 * Forces CPU into sleep state
 *
 * omap1510_idle_loop_suspend()
 *
 * cpu_omap_suspend()'s resume point.
 *
 * Note: This code get's copied to SRAM at boot.  When the omap
 *	wakes up it continues execution at the point it went to sleep.
 *	It will just start executing here, so we'll restore stuff from the
 *	stack, reset the ARM_IDLECT1 and ARM_IDLECT2 and return 
 */

ENTRY(omap1510_idle_loop_suspend)

	stmfd	sp!, {r0 - r12, lr}		@ save registers on stack

	@ turn off clock domains
	@ get arm_idlect2 into r1
	ldr	r4, =ARM_ASM_IDLECT2
	ldrh	r1, [r4]
	@ ldr	r5, =IDLE_CLOCK_DOMAINS
	@ strh	r5, [r4]

	@ request ARM idle
	@ get arm_idlect1 into r1
	ldr	r4, =ARM_ASM_IDLECT1
	ldrh	r2, [r4]
	ldr	r3, =IDLE_LOOP_REQUEST
	orr	r3, r3, r2
	strh	r3, [r4]

	mov	r4, #0
	ldr	r5, =IDLE_WAIT_CYCLES
loop:
	add	r4, r4, #1
	cmp	r4, r5
	bne	loop
	nop
/*
 * Let's wait for the next clock tick to wake us up.
 * If we do both request sleep and wait for interrupt
 * instruction it takes 2 interrupts to actually wake us
 * from here.  So we don't do the mcr instruction
 */
@	mcr	p15, 0, r0, c7, c0, 4		@ wait for interrupt
/*
 * cpu_omap_suspend()'s resume point.
 *
 * Note: This code get's copied to SRAM at boot.  When the omap
 *	wakes up it continues execution at the point it went to sleep.
 *	It will just start executing here, so we'll restore stuff from the
 *	stack, reset the ARM_IDLECT1 and ARM_IDLECT2 and return to
 *	the address passed in arg0.
 */

	@ restore ARM_IDLECT1 and ARM_IDLECT2 and return
	@ r2 has arm_idlect1 and r1 still has arm_idlect2
	ldr	r4, =ARM_ASM_IDLECT2
	strh	r1, [r4]

	ldr	r4, =ARM_ASM_IDLECT1
	strh	r2, [r4]

	ldmfd   sp!, {r0 - r12, pc}     @ restore regs and return
	nop


ENTRY(omap1510_cpu_suspend)

	stmfd	sp!, {r0 - r12, lr}		@ save registers on stack

	@ prepare to put SDRAM into self-refresh manually
	ldr	r4, =EMIFF_ASM_SDRAM_CONFIG
	ldr	r5, [r4]
	ldr	r3, =SELF_REFRESH_MODE
	orr	r5, r5, r3
	str	r5, [r4]

	@ prepare to put EMIFS to Sleep
	ldr	r4, =EMIFS_ASM_CONFIG_REG
	ldr	r5, [r4]
	ldr	r3, =IDLE_EMIFS_REQUEST
	orr	r5, r5, r3
	str	r5, [r4]

	@ turn off clock domains
	ldr	r4, =ARM_ASM_IDLECT2
	ldrh	r1, [r4]
	ldr	r5, =IDLE_CLOCK_DOMAINS
	strh	r5, [r4]

	@ request ARM idle
	ldr	r4, =ARM_ASM_IDLECT1
	ldrh	r2, [r4]
	ldr	r3, =BIG_SLEEP_REQUEST
	strh	r3, [r4]
/*
 * The next UART interrupt will awaken it.
 */
	mov	r4, #0
	ldr	r5, =IDLE_WAIT_CYCLES
loop2:
	add	r4, r4, #1
	cmp	r4, r5
	bne	loop2
	nop
	mcr	p15, 0, r0, c7, c0, 4		@ wait for interrupt

	ldr	r4, =ARM_ASM_IDLECT2
	strh	r1, [r4]

	ldr	r4, =ARM_ASM_IDLECT1
	strh	r2, [r4]

	ldmfd   sp!, {r0 - r12, pc}     @ restore regs and return
	nop


