#ifndef TX4925_TX4925_NAND_H
#define TX4925_TX4925_NAND_H

/*
 * linux/include/asm-mips/tx4925/tx4925_nand.h
 *
 * tx4925_nand defs
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <asm/tx4925/tx4925.h>

#define tx4925_read_nfmc(addr) (*(volatile unsigned int *)(addr))
#define tx4925_write_nfmc(b,addr) (*(volatile unsigned int *)(addr)) = (b)

/* TX4925 NAND Flash Memory Mode Control Register (NDFMCR) bits */
#define TX4925_NDFMCR_WE              0x80 /* Write Enable         */
#define TX4925_NDFMCR_ECC_CNTL_MASK   0x60 /* ECC Control          */
#define TX4925_NDFMCR_ECC_CNTL_ENAB   0x20 /* ECC Control          */
#define TX4925_NDFMCR_ECC_CNTL_READ   0x40 /* ECC Control          */
#define TX4925_NDFMCR_ECC_CNTL_DISAB  0x00 /* ECC Control          */
#define TX4925_NDFMCR_CE              0x10 /* Chip Enable          */
#define TX4925_NDFMCR_RESERVED        0x08 /* Reserved             */
#define TX4925_NDFMCR_BSPRT           0x04 /* Bus Seperate         */
#define TX4925_NDFMCR_ALE             0x02 /* Address Latch Enable */
#define TX4925_NDFMCR_CLE             0x01 /* Command Latch Enable */

/* TX4925 NAND Flash Memory Status Register (NDFSR) bits */
#define TX4925_NDSFR_BUSY             0x80 /* 0: Ready 1: Busy */

/* TX4925 NAND Flash Memory Interrupt Status Register (NDFISR) bits */
#define TX4925_NDFISR_RDY             0x01 /* Ready */

/* TX4925 NAND Flash Memory Interrupt Mask Register (NDFIMR) bits */
#define TX4925_NDFIMR_INTEN           0x80 /* Interrupt Enable */
#define TX4925_NDFIMR_MRDY            0x01 /* Mask Ready Interupt */

/* TX4925 NAND Flash Memory Reset Register (NDFRSTR) bits */
#define TX4925_NDFRSTR_RST            0x01 /* Reset */

struct tx4925_ndfmc_reg {
       volatile unsigned long dtr;
       volatile unsigned long mcr;
       volatile unsigned long sr;
       volatile unsigned long isr;
       volatile unsigned long imr;
       volatile unsigned long spr;
       volatile unsigned long rstr;
};                                       

#define TX4925_HOLD           2
#define TX4925_SPW            6
#define TX4925_BSPRT          1

#ifndef _LANGUAGE_ASSEMBLY

#define tx4925_ndfmcptr	  ((struct tx4925_ndfmc_reg *)TX4925_REG(TX4925_NDFMC_BASE))

#endif /* _LANGUAGE_ASSEMBLY */

#endif /*  TX4925_NAND_H */
