/*
 * FILE NAME
 *	include/asm-mips/vr4181a/vr4181a.h
 *
 * BRIEF MODULE DESCRIPTION
 *	Include for NEC VR4181A.
 *
 * Author: Yoichi Yuasa
 *         yyuasa@mvista.com or source@mvista.com
 *
 * Copyright 2002,2003 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __NEC_VR4181A
#define __NEC_VR4181A

#include <linux/interrupt.h>

/*
 * Interrupt Numbers
 */
#define USBHU_IRQ	36
#define USBFU_IRQ	37
#define AC97U_IRQ	38

/*
 * Internal Registers access
 */
extern unsigned long vr4181a_internal_registers_base;

#define set_vr4181a_internal_registers_base(base)  \
	do { vr4181a_internal_registers_base = (base); } while (0)

#define vr4181a_readb(offset)	\
	(*(volatile unsigned char *)(vr4181a_internal_registers_base + (offset)))
#define vr4181a_readw(offset)	\
	(*(volatile unsigned short *)(vr4181a_internal_registers_base + (offset)))
#define vr4181a_readl(offset)	\
	(*(volatile unsigned int *)(vr4181a_internal_registers_base + (offset)))

#define vr4181a_writeb(b,offset)	\
	(*(volatile unsigned char *)(vr4181a_internal_registers_base + (offset))) = (b)
#define vr4181a_writew(b,offset)	\
	(*(volatile unsigned short *)(vr4181a_internal_registers_base + (offset))) = (b)
#define vr4181a_writel(b,offset)	\
	(*(volatile unsigned int *)(vr4181a_internal_registers_base + (offset))) = (b)

/*
 * Basic Timer
 */
extern void vr4181a_timer_setup(struct irqaction *irq);

/*
 * Architecture-specific implementations of sys_reboot commands.
 */
extern void vr4181a_restart(char *command);
extern void vr4181a_halt(void);
extern void vr4181a_power_off(void);

/*
 * Clock Control Unit
 */
extern void vr4181a_ccu_init(unsigned long ecu_sysclock);

extern void vr4181a_clock_supply(unsigned short clock);
extern void vr4181a_clock_mask(unsigned short clock);

enum {
	ADU18M_CLOCK,
	ADU_CLOCK,
	PIU_CLOCK,
	AIU_CLOCK,
	CSU_CLOCK,
	SIU0_CLOCK,
	SIU1_CLOCK,
	SIU2_CLOCK,
	PWM2_CLOCK,
	AC97_CLOCK,
	I2C_CLOCK,
	USB_HOST_CLOCK,
	USB_FUNCTION_CLOCK,
	I2S_CLOCK,
	PWM01_CLOCK,
	DMA_CLOCK,
	PCI_BRIDGE_CLOCK,
	LCD_CLOCK,
	USB_HOST_PCI_CLOCK,
	USB_FUNCTION_PCI_CLOCK,
	AC97_PCI_CLOCK,
};

/*
 * Host Bridge Unit
 */
struct vr4181a_address_window {
	unsigned long base_address;
	unsigned long size;
	unsigned char data_width;
	unsigned char pci_master_access;
};

enum {
	DATA_WIDTH_8BIT,
	DATA_WIDTH_16BIT,
	DATA_WIDTH_32BIT,
};

enum {
	PCI_MASTER_ACCESS_PROHIBIT,
	PCI_MASTER_ACCESS_ENABLE,
};

struct vr4181a_address_maps {
	struct vr4181a_address_window dram;
	struct vr4181a_address_window sync_flash_memory;
	struct vr4181a_address_window pcs0;
	struct vr4181a_address_window pcs1;
	struct vr4181a_address_window pcs2;
	struct vr4181a_address_window pcs3;
	struct vr4181a_address_window pcs4;
	struct vr4181a_address_window isa_bus;
	struct vr4181a_address_window pci_bus0;
	struct vr4181a_address_window pci_bus1;
	struct vr4181a_address_window internal_registers;
	struct vr4181a_address_window rom;
};

extern void vr4181a_hbu_init(struct vr4181a_address_maps *maps);

/*
 * External System Bus Control Unit
 */
struct vr4181a_access_config {
	unsigned char io_signal_setup_time;
	unsigned char page_access_length;
	unsigned char control_signal_active_level;
	unsigned char chip_select_active_level;
	unsigned char control_signal_deassert_timing;
	unsigned char iordy_sync_times;
	unsigned char ready_mode;
	unsigned char bus_idel_time;
	unsigned char control_signal_width_min;
	unsigned char control_signal_hold_time;
	unsigned char control_signal_assert_time_page_access;
	unsigned char control_signal_assert_time;
	unsigned short ready_signal_timeout_counter;
	unsigned char control_signal_setup_time;
	unsigned char config_active;
};

enum {
	ACTIVE_LOW,
	ACTIVE_HIGH,
};

enum {
	LATCH_1_TIME,
	LATCH_2_TIMES,
};

enum {
	READY_UNUSE,
	READY_USE,
};

struct vr4181a_access_setup {
	struct vr4181a_access_config pcs0;
	struct vr4181a_access_config pcs1;
	struct vr4181a_access_config pcs2;
	struct vr4181a_access_config pcs3;
	struct vr4181a_access_config pcs4;
	struct vr4181a_access_config isa_bus;
	struct vr4181a_access_config rom;
};

extern void vr4181a_exibu_init(struct vr4181a_access_setup *setup,
                               unsigned long lclock_max);

/*
 * Internal PCI Bridge Unit
 */
struct vr4181a_pci_base_address {
	unsigned long pci_base_address;
	unsigned char prefetch;
	unsigned char address_active;
};

enum {
	PREFETCH_PROHIBIT,
	PREFETCH_ENABLE,
};

struct vr4181a_pci_window_config {
	unsigned long pci_base_address;
	unsigned char access_width;
	unsigned char command_type;
};

enum {
	ACCESS_WIDTH_8BIT,
	ACCESS_WIDTH_16BIT,
	ACCESS_WIDTH_32BIT,
};

enum {
	COMMAND_TYPE_IO,
	COMMAND_TYPE_MEMORY,
	COMMAND_TYPE_CONFIGURATION,
};

struct vr4181a_pci_config {
	struct vr4181a_pci_base_address internal_registers;
	struct vr4181a_pci_base_address dram;
	struct vr4181a_pci_base_address sync_flash_memory;
	struct vr4181a_pci_base_address pcs0;
	struct vr4181a_pci_base_address pcs1;
	struct vr4181a_pci_base_address pcs2;
	struct vr4181a_pci_base_address pcs3;
	struct vr4181a_pci_base_address pcs4;
	struct vr4181a_pci_base_address isa_bus;
	struct vr4181a_pci_base_address rom;
	struct vr4181a_pci_window_config pci_window0;
	struct vr4181a_pci_window_config pci_window1;
	unsigned char master_latency;
	unsigned short delayed_transaction_abort;
	unsigned short retry_limit;
};

extern struct pci_ops vr4181a_pci_ops;

extern void vr4181a_iopciu_init(struct vr4181a_pci_config *config);

/*
 * Power Management Unit
 */
extern void vr4181a_reset_haltimer(void);
extern void vr4181a_soft_reset(void);
extern void vr4181a_wait(void);
extern void vr4181a_change_divide(int mode, int action);

enum {
	DIVIDE_DEFAULT,
	DIVIDE_MODE2,
	DIVIDE_MODE3,
};

enum {
	SOFT_RESET,
	HIBERNATE,
};

/*
 * Serial Interface Unit
 */
extern void vr4181a_siu_init(unsigned char unit, int line);

/*
 * General-purpose I/O Unit
 */
extern void vr4181a_set_irq_trigger(unsigned short pin, int trigger);

enum {
	TRIGGER_EDGE,
	TRIGGER_LEVEL,
};

extern void vr4181a_set_irq_level(unsigned short pin, int level);

enum {
	LEVEL_LOW,
	LEVEL_HIGH,
};

extern int vr4181a_set_gpio_mode(unsigned short pin, int mode, int input_mode);

enum {
	GPIO_INPUT,
	GPIO_OUTPUT,
	GPIO_OTHER,
};

enum {
	GPIO_INPUT_PROHIBIT,
	GPIO_INPUT_ENABLE,
};

extern int vr4181a_get_gpio_data(unsigned short pin);
extern int vr4181a_set_gpio_data(unsigned short pin , int data);

extern u16 vr4181a_read_nvreg(int regno);
extern void vr4181a_write_nvreg(int regno, u16 value);

extern void vr4181a_set_pinmode(int mode);

enum {
	PINMODE_ECU1,
	PINMODE_PWM0,
	PINMODE_PWM1,
	PINMODE_PWM2,
	PINMODE_KIU,
	PINMODE_LCU,
	PINMODE_SIU0,
	PINMODE_SIU1,
	PINMODE_SIU2,
	PINMODE_USB_ENABLE,
	PINMODE_USB_PROHIBIT,
};

extern void vr4181a_power_active_usb_function(void);
extern void vr4181a_power_save_usb_function(void);

extern void vr4181a_power_active_usb_host(void);
extern void vr4181a_power_save_usb_host(void);

#endif /* __NEC_VR4181A */
