#!/bin/bash

# directory where temporary ramdisk & cramfs directories have to be created
# and where the INITRD_IMG file from targetconf can be found.
INITRD_SRCDIR=`pwd`

# temporary ramdisk directory name
INITRD_MNT_DIR=rdiskdir

# temporary cramfs directory name
CRAMFS_DIR=cramfs

# directory where image need to be created
GZIP_IMG_DIR=$LINUX_SRC

#igateway directory from where objects need to be copied to ramdisk &/or cramfs
IGW_DIR=$GZIP_IMG_DIR/../igw/igateway/system/zcomixp 

#final initrd image name
INITRD_IMG=initrd.o

#disk image name
DISK_IMG=disk.img

#cramfs image name
CRAMFS_IMG=cramfs.img

#name of tar file created by targetconf
TAR_INITRD_IMG=fsimage.tar 

#image name for gzipped ramdisk image
GZIP_INITRD_IMG=ramdisk.gz 

#linker command definition
LD=xscale_be-ld

#linux directory
LINUX_DIR=`pwd`

#csr directorywhere csr.o was built
CSR_DIR=$IX_XSCALE_SW/lib/linuxobjs

###################################################
INST_RDISK_DIR=$INITRD_SRCDIR/$INITRD_MNT_DIR
INST_CRAMFS_DIR=$INITRD_SRCDIR/$CRAMFS_DIR
export INST_RDISK_DIR INST_CRAMFS_DIR IGW_DIR

CWD=`pwd`

# create temporary ramdisk & cramfs directories
echo "creating temp ramdisk dir"
cd $INITRD_SRCDIR
if [ -d $INITRD_MNT_DIR ]
then
rm -rf $INITRD_MNT_DIR
fi
mkdir $INITRD_MNT_DIR
chmod 777 $INITRD_MNT_DIR

#create cramfs directory if it does not exist, clear contents anyway
echo "creating temp cramfs dir"
if [ -d $CRAMFS_DIR ]
then
rm -rf $CRAMFS_DIR
fi
mkdir $CRAMFS_DIR
chmod 777 $CRAMFS_DIR

#copy ethernet drivers & other insertable modules to cramfs
cp $IX_XSCALE_SW/lib/linuxobjs/csr.o $CRAMFS_DIR/.
cp $IX_XSCALE_SW/lib/linuxobjs/ixp425_eth.o $CRAMFS_DIR/.
#cp $LINUX_DIR/drivers/net/ixp425_eth.o $CRAMFS_DIR/.
#cp $LINUX_DIR/drivers/net/eepro100.o $CRAMFS_DIR/.

#Chuck
cp $HARDHAT_BASE/devkit/arm/xscale_be/target/usr/sbin/eraseall $CRAMFS_DIR/.

# create a initrd image
dd if=/dev/zero of=$DISK_IMG bs=1024 count=4000

#make an ext2 file system on it
/sbin/mke2fs -F $DISK_IMG

# mount this image
mount -oloop $DISK_IMG $INITRD_SRCDIR/$INITRD_MNT_DIR

#untar the image created using targetconf utility
echo "creating rdiskimg"
cd $INITRD_SRCDIR/$INITRD_MNT_DIR
tar -xf $INITRD_SRCDIR/$TAR_INITRD_IMG

# change all executable files in ramdisk bin directory to soft links 
# to busybox in cramfs directory
cd bin
for filename in *  
do
if [ -f $filename ]
then
cp $filename ../../$CRAMFS_DIR/.
rm $filename
ln -s /mnt/cramfs/$filename $filename
fi
done

#Chuck : restore all command links to busybox
cd ../sbin
for filename in *  
do
if [ -f $filename ]
then
#cp $filename ../../$CRAMFS_DIR/.
rm $filename
#ln -s /mnt/cramfs/$filename $filename
ln -s /bin/busybox $filename 
fi
done

#mtd devices required
cd ../dev
for i in 0 1 2 3 4
do
j=`expr $i \* 2`
mknod mtd$i c 90 $j
mknod mtdblock$i b 31 $i
done

#create cramfs directory
cd ../mnt
mkdir cramfs

#copy files from igateway to ramdisk & cramfs
cd $GZIP_IMG_DIR
./CpIgw

#unmount ramdisk image
umount $INITRD_SRCDIR/$INITRD_MNT_DIR

cd $GZIP_IMG_DIR

#remove old ramdisk image
if [ -f $GZIP_INITRD_IMG ]
then
rm $GZIP_INITRD_IMG
fi

#zip the ramdisk image
gzip -9 -c $DISK_IMG > $INITRD_IMG
rm $DISK_IMG

#convert to .o
$LD -s -r -o img.o -b binary $INITRD_IMG

mv img.o $INITRD_IMG 

